/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include <DxcConfig.hpp>
#include <Connection/ScenarioRecorderConnector.hpp>

using namespace Dxc;

ScenarioRecorderConnector::ScenarioRecorderConnector(DxcCallback *cb)
    : DxcConnectorImp(cb),
      sdsock_(-1),
      dasock_(-1),
      slsock_(-1),
      orsock_(-1)
{
    initConnections();
    acceptConnections();
}

void ScenarioRecorderConnector::initConnections()
{
    DxcConnectorImp::connect("SDS", "localhost", DxcConfig::getInstance()->getInt("SDS_SR_PORT"), &sdsock_);
}

bool ScenarioRecorderConnector::sendMessage(const DxcData * msg)
{
    bool ret = false;
    if (typeid(*msg).name() == typeid(CommandData).name() ||
            typeid(*msg).name() != typeid(ScenarioData).name() ||
            typeid(*msg).name() != typeid(ScenarioStatusData).name() ||
            typeid(*msg).name() != typeid(RecoveryData).name())
    {
        if (msg->getSource() != "OR" && orsock_ != -1)
        {
            ret = DxcConnectorImp::transmit(orsock_, msg);
        }
    }

    if (typeid(*msg).name() == typeid(ScenarioStatusData).name())
        ret &= DxcConnectorImp::transmit(sdsock_, msg);

    if (typeid(*msg).name() == typeid(RecoveryData).name() &&
            msg->getSource() == "OR")
        ret &= DxcConnectorImp::transmit(dasock_, msg);

    return ret;
}

void ScenarioRecorderConnector::acceptConnections()
{
    DxcConnectorImp::acceptConnection("DA", DxcConfig::getInstance()->getInt("SR_DA_PORT"), &dasock_);
    DxcConnectorImp::acceptConnection("SL", DxcConfig::getInstance()->getInt("SR_SL_PORT"), &slsock_);
    if (DxcConfig::getInstance()->get("RUN_ORACLE") == "TRUE")
        DxcConnectorImp::acceptConnection("OR", DxcConfig::getInstance()->getInt("SR_OR_PORT"), &orsock_);
}
