/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include "Candidate.hpp"

using namespace std;
using namespace Dxc;

Candidate::Candidate() : faults_(), weight_(-1)
{
}

Candidate::Candidate(const FaultSet& faults, double optionalWeight)
    : faults_(faults), weight_(optionalWeight)
{
}

bool Candidate::operator<(const Candidate& c1) const
{
    if (this->faults_ == c1.faults_ &&
            this->weight_ < c1.weight_) return true;
    if (this->faults_ < c1.faults_) return true;
    return false;
}

FaultSet* Candidate::getFaults()
{
    return &faults_;
}

void Candidate::setFaults(const FaultSet& f)
{
    faults_ = f;
}

double Candidate::getWeight() const
{
    return weight_;
}

void Candidate::setWeight(double w)
{
    weight_ = w;
}

std::ostream &Candidate::put(std::ostream &out) const
{
    out << faults_ << " [weight = "
        << this->getWeight() << "]";

    return out;
}

ostream & operator << (ostream &out, const Dxc::Candidate& c)
{
    c.put(out);
    return out;
}

std::ostream &CandidateSet::put(std::ostream &out) const
{
    out << "{ ";

    if (!this->empty())
    {
        bool fg = false;
        for (CandidateSet::const_iterator i = this->begin();
                i != this->end();
                i++)
        {
            if (fg)
            {
                out << ", ";
            }
            else
            {
                fg = true;
            }
            out << *i;
        }
    }

    out << " }";

    return out;

}

ostream & operator << (ostream &out, const Dxc::CandidateSet& cs)
{
    cs.put(out);
    return out;
}
