/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include "Command.hpp"

using namespace std;
using namespace Dxc;

Command::Command(const string& commandID, const Value* value)
    : commandID_(commandID), value_(value->clone())
{
}

string Command::getCommandID() const
{
    return commandID_;
}

const Value* Command::getValue() const
{
    return value_;
}

ostream &Command::put(ostream & out) const
{
    out << commandID_ << " = ";
    value_->put(out);

    return out;
}

bool Command::operator<(const Command& f1) const
{
    if( this->getCommandID() == f1.getCommandID() &&
            this->getValue()->lt(f1.getValue())) return true;
    if( this->getCommandID() < f1.getCommandID() ) return true;
    return false;
}

bool Command::operator==(const Command& f1) const
{
    if( this->getCommandID() != f1.getCommandID() ||
            !(this->getValue()->eq(f1.getValue()))) return false;
    return true;
}

ostream & operator << (ostream &out, const Command& f)
{
    return f.put(out);
}

ostream &CommandSet::put(ostream &out) const
{
    out << "{ ";

    if (!this->empty())
    {
        bool fg = false;
        for (CommandSet::const_iterator i = this->begin();
                i != this->end();
                i++)
        {
            if (fg)
            {
                out << ", ";
            }
            else
            {
                fg = true;
            }
            out << *i;
        }
    }

    out << " }";

    return out;
}

ostream & operator << (ostream &out, const Dxc::CommandSet& fs)
{
    fs.put(out);
    return out;
}


Command::Command(const Command& f)
{
    commandID_ = f.getCommandID();
    value_ = f.getValue()->clone();
}

Command& Command::operator=(const Command& f)
{
    if (this != &f)
    {
        commandID_ = f.getCommandID();
        value_ = f.getValue()->clone();
    }

    return *this;
}

Command::~Command()
{
    delete value_;
}
