package org.dxc.api.connection;

import static org.junit.Assert.*;

import org.junit.Test;
import org.dxc.api.DxcCallback;
import org.dxc.api.datatypes.*;

public class TestToOracleConnector {
    private class MessageHandler extends DxcCallback {
        private DxcData d;

        public MessageHandler() {
            d = null;
        }

        public void processData(DxcData d) {
            if (this.d != null)
                fail("MessageHandler received more than one message.");
            else
                this.d = d;
        }

        public DxcData getMessage() {
            if (d == null) {
                fail("MessageHandler received no message.");
                return null;
            } else {
                return d;
            }
        }
    }

    @Test
    public void messageTo() throws Exception {
        MessageHandler mhTo = new MessageHandler();
        DxcConnector toc = ConnectorFactory.getToOracleConnector(null);
        Thread.sleep(10);
        DxcConnector foc = ConnectorFactory.getFromOracleConnector(mhTo);
        Thread.sleep(10);

        CommandData cd = new CommandData("test", new StringValue("test1"));

        toc.sendMessage(cd);
        Thread.sleep(100);
        assertTrue(cd.equals(mhTo.getMessage()));

        toc.shutdown();
        foc.shutdown();
    }

    @Test
    public void messageFrom() throws Exception {
        MessageHandler mhFrom = new MessageHandler();
        DxcConnector toc = ConnectorFactory.getToOracleConnector(mhFrom);
        Thread.sleep(10);
        DxcConnector foc = ConnectorFactory.getFromOracleConnector(null);
        Thread.sleep(10);

        CommandData cd = new CommandData("test", new StringValue("test1"));

        foc.sendMessage(cd);
        Thread.sleep(100);
        assertTrue(cd.equals(mhFrom.getMessage()));

        toc.shutdown();
        foc.shutdown();
    }

}