/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.evaluator;

import org.dxc.api.datatypes.*;
import org.dxc.api.DxcCallback;
import org.dxc.api.DxcDataGenerator;

import java.util.HashMap;
import java.util.Map;

import org.dxc.evaluator.metrics.*;

public class ScenarioResultData extends DxcCallback implements ScenarioResult {
    private Map<String, ScenarioMetricCalculator> metricCalcs;
    private boolean hasErrors;

    public ScenarioResultData() throws Exception {
        EvalConfig ec = EvalConfig.getInstance();
        metricCalcs = new HashMap<String, ScenarioMetricCalculator>();

        for (MetricInfo m : ec.getMetrics()) {
            String name = m.getScenarioMetricName();
            if (!metricCalcs.containsKey(name)) {
                ScenarioMetricCalculator mc = MetricCalculatorFactory
                        .getScenarioMetric(name);
                if (mc != null)
                    metricCalcs.put(name, mc);
            }
        }
    }

    public ScenarioResultData(DxcDataGenerator dgen) throws Exception {
        this();

        process(dgen);
    }

    protected void process(DxcDataGenerator dgen) throws Exception {
        if (dgen != null)
            dgen.begin(this);

        for (ScenarioMetricCalculator mc : metricCalcs.values()) {
            mc.calculate();
        }
    }

    public void processData(DxcData data) {
        for (ScenarioMetricCalculator mc : metricCalcs.values()) {
            mc.addData(data);
        }

        if (!hasErrors && data instanceof ErrorData)
            hasErrors = true;
    }

    public double getMetricScore(String metric) {
        double ret = Double.NaN;
        ScenarioMetricCalculator smc = metricCalcs.get(metric);
        if (smc != null)
            ret = smc.getScore();

        return ret;
    }

    public String getName() {
        return "DxcDataGenerator scenario";
    }

    public boolean hasErrors() {
        return hasErrors;
    }

    public void print() {
        System.out.println("Scenario: " + this.getName());

        for (String s : metricCalcs.keySet())
            metricCalcs.get(s).print();

        System.out.println();
    }

    public ScenarioMetricCalculator getCalculator(String metricName) {
        return metricCalcs.get(metricName);
    }
}
