package org.dxc.evaluator.metrics;

import org.dxc.api.datatypes.Candidate;
import org.dxc.api.datatypes.DiagnosisData;
import org.dxc.api.datatypes.DxcData;
import org.dxc.api.datatypes.FaultSet;
import org.dxc.api.datatypes.ScenarioData;
import org.dxc.api.datatypes.FaultInjectData;
import org.dxc.api.datatypes.SystemCatalog;
import org.dxc.api.datatypes.SystemCatalogXML;

/*
 * CPU Time metric.
 */
public class Utility implements ScenarioMetricCalculator {
    private double utility = Double.NaN;
    private FaultSet injection = new FaultSet();
    private DiagnosisData lastDiagnosis;
    int faultableComponents;

    public void addData(DxcData d) {
        if (d instanceof FaultInjectData) {
            FaultInjectData fid = (FaultInjectData) d;
            ;
            if (fid.isInjection()) {
                injection.add(fid.getFault());
            } else {
                injection.remove(fid.getFault());
            }
        } else if (d instanceof DiagnosisData) {
            DiagnosisData dd = (DiagnosisData) d;
            if (lastDiagnosis != null) {
                if (!lastDiagnosis.getCandidateSet().equals(
                        dd.getCandidateSet()))
                    lastDiagnosis = dd;
            } else {
                lastDiagnosis = dd;
            }
        } else if (d instanceof ScenarioData) {
            ScenarioData sd = (ScenarioData) d;
            SystemCatalog sys = null;
            try {
                sys = SystemCatalogXML.getInstance(sd.getSystemName());
                if (sys != null)
                    faultableComponents = sys.getComponentIDs().size()
                            - sys.getNumNoFaultComponents();
            } catch (Exception ex) {
                faultableComponents = -1;
            }
        }
    }

    public void calculate() {
        if (faultableComponents < 1)
            return;

        if (lastDiagnosis == null)
            lastDiagnosis = new DiagnosisData(false, false, null, null);

        for (Candidate c : lastDiagnosis.getCandidateSet()) {
            FaultSet diagnosis = c.getFaultSet();

            FaultSet falseNegatives = new FaultSet(injection);
            falseNegatives.removeAll(diagnosis);
            FaultSet falsePositives = new FaultSet(diagnosis);

            falsePositives.removeAll(injection);

            double w = c.getWeight();
            int healthyGuess = faultableComponents - diagnosis.size();
            int faultyGuess = diagnosis.size();
            double Ep = calculateE(falsePositives.size(), faultyGuess);
            double En = calculateE(falseNegatives.size(), healthyGuess);
            double u = 1.0 - 0.5 * (Ep + En) / faultableComponents;

            if (Double.isNaN(utility))
                utility = u * w;
            else
                utility += u * w;
        }
    }

    private double calculateE(int n, int N) {
        return (n * (N - n + 1.0) / (n + 1.0));
    }

    public double getScore() {
        return utility;
    }

    public void print() {
        System.out.println("Utility: " + utility);
    }
}