/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include <stdio.h>
#include <malloc.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>

#include <cstdlib>
#include <string>

using namespace std;

static void DxcInitHook();
static void *DxcMallocHook(size_t, const void *);
static void *DxcReallocHook(void *, size_t, const void *);
static void *DxcMemalignHook(size_t, size_t, const void *);

void (* __malloc_initialize_hook)() = DxcInitHook;

void * (* oldMallocHook)(size_t, const void *);
void * (* oldReallocHook)(void *, size_t, const void *);
void * (* oldMemalignHook)(size_t, size_t, const void *);

static long oldMemUse = 0L;

static key_t key;
static int shmid;

static void DxcGetMemUse()
{
    struct mallinfo m = mallinfo();

    long memUse = m.hblkhd + m.uordblks;
    if (memUse > oldMemUse)
    {
        long *memuse = (long *)shmat(shmid, NULL, 0);
        *memuse += (memUse - oldMemUse);
        shmdt(memuse);

        oldMemUse = memUse;
        /*
                fprintf(stderr, "[pid %d] %ld\n", getpid(), memUse);
        */
    }
}

static void DxcInitHook()
{
    char *dxcHome = getenv("DXC_HOME");
    string dxcMemProfLib = string(dxcHome) + "/Lib/libdxcMemProf.so";

    key = ftok(dxcMemProfLib.c_str(), 0);
    shmid = shmget(key, sizeof(long), 0644);

    oldMallocHook = __malloc_hook;

    __malloc_hook = DxcMallocHook;
}

static void *DxcMallocHook(size_t size, const void *caller)
{
    void *result;

    __malloc_hook = oldMallocHook;

    result = malloc(size);

    DxcGetMemUse();
    /*
        fprintf(stderr, "[pid %d] malloc (%u) returns %p\n", getpid(), (unsigned int)size, result);
    */
    __malloc_hook = DxcMallocHook;

    return result;
}

static void *DxcReallocHook(void *ptr, size_t size, const void *caller)
{
    void *result;

    __realloc_hook = oldReallocHook;

    result = realloc(ptr, size);

    DxcGetMemUse();

    __realloc_hook = DxcReallocHook;

    return result;
}

static void *DxcMemalignHook(size_t alignment, size_t size, const void *caller)
{
    void *result;

    __memalign_hook = oldMemalignHook;

    result = memalign(alignment, size);

    DxcGetMemUse();

    __memalign_hook = DxcMemalignHook;

    return result;
}
